/*
   Copyright (c) 2014-2019 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */


#include "includes.h"

#ifndef HAL_H_
#define HAL_H_

#include "dio.h"
#include "ain.h"
#include "pwm.h"
#include "qep.h"
#include "spi.h"
#include "power.h"

#include "hal_impl.h"

extern void HAL_sinit();
extern void HAL_configure();

// digital I/O
extern void HAL_setupDigitalOut(uint16_t aChannel, uint16_t aResourceNumber, bool aInitialValue);
extern void HAL_setupDigitalIn(uint16_t aChannel, uint16_t aResourceNumber);
extern void HAL_setDigitalOut(uint16_t aChannel, bool aVal);
extern bool HAL_getDigitalIn(uint16_t aChannel);

// analog in
extern AIN_AdcParams_t* HAL_getDefaultAdcParams();
extern void HAL_setupAdc(uint16_t aHandle, uint16_t aAdc, const AIN_AdcParams_t *aInParams);
extern AIN_ChannelParams_t* HAL_getDefaultAinChannelParams();
extern void HAL_setupAnalogIn(uint16_t aHandle, uint16_t aChannel, uint16_t aResourceNumber,  const AIN_ChannelParams_t *aParams);
extern void HAL_setupAnalogInF(uint16_t aHandle, uint16_t aChannel, uint16_t aResourceNumber, const AIN_ChannelParams_t *aParams, float scale, float offset);
extern uint16_t HAL_getAnalogIn(uint16_t aHandle, uint16_t aChannel);
extern float HAL_getAnalogInF(uint16_t aHandle, uint16_t aChannel);

// PWM
extern PWM_Params_t* HAL_getDefaultPwmParams(uint16_t aPeriodCounts, uint16_t aDeadtimeCounts, uint16_t aSOCASelect);
extern void HAL_setupPwm(uint16_t aChannel, uint16_t aResourceNumber, const PWM_Params_t *aPwmParams);
extern void HAL_setPwm(uint16_t aChannel, uint16_t aVal);
extern void HAL_setPwmDuty(uint16_t aChannel, float aDuty);
extern void HAL_setPwmPh(uint16_t aChannel, float aDuty, float aPhase, uint16_t aCntrMode);
extern uint32_t HAL_getPwmPeriod(uint16_t aChannel);
extern uint32_t HAL_getPwmCounter(uint16_t aChannel);
extern bool HAL_getPwmCountDirection(uint16_t aChannel);
extern void HAL_setupPwmSyncSelect(uint16_t aResourceNumber, uint16_t aType, uint16_t aUnit);

// powerstage
extern void HAL_enablePower(bool aEnable);
extern bool HAL_powerIsEnabled();
extern void HAL_setupPwrTzGpio(uint16_t aTzId, uint16_t aGpio);

// QEP
extern QEP_Params_t* HAL_getDefaultQepParams(uint32_t aPeriodCounts, bool aEnableResetByIndexPulse);
extern void HAL_setupQep(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aPinSet, QEP_Params_t *aParams);
extern uint32_t HAL_getQepCounter(uint16_t aChannel);
extern bool HAL_getAndClearQepIndexFlag(uint16_t aChannel);
extern uint32_t HAL_getQepIndexLatchCounter(uint16_t aChannel);

// SPI
extern SPI_Params_t* HAL_getDefaultSpiParams(uint16_t aCharLength, bool aPol, bool aPhase, uint32_t aBaudRate, bool aMasterSlave);
extern void HAL_setupSpi(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aPinSet, SPI_Params_t *aParams);
extern void HAL_setSpiMasterOut(uint16_t aChannel, uint16_t aChar);
extern uint16_t HAL_getSpiMasterIn(uint16_t aChannel);
extern uint16_t HAL_getSpiSlaveIn(uint16_t aChannel);
extern void HAL_setSpiSlaveOut(uint16_t aChannel, uint16_t aChar);

extern uint16_t HAL_getSpiTxFifoLevel(int16_t aChannel);
extern uint16_t HAL_getSpiRxFifoLevel(int16_t aChannel);

extern void HAL_resetSpiFifos(int16_t aChannel);
extern void HAL_putSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen);
extern void HAL_getSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen);
extern void HAL_putGetSpiWords(int16_t aChannel, uint16_t *aDataOut, uint16_t *aDataIn, uint16_t aLen);

#endif /* HAL_H_ */
