/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "spi.h"

#pragma diag_suppress 112 // PLX_ASSERT(0) in switch statements

SPI_Handle_t SPI_init(void *aMemory, const size_t aNumBytes)
{
	if(aNumBytes < sizeof(SPI_Obj_t))
	{
		return((SPI_Handle_t)NULL);
	}
	SPI_Handle_t handle = (SPI_Handle_t)aMemory;
	return handle;
}

void SPI_configure(SPI_Handle_t aHandle, uint16_t port, uint32_t clk)
{
	SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;
	obj->port = port;
	obj->clk = clk;
}

void SPI_setupPort(SPI_Handle_t aHandle, uint16_t aPinSet, SPI_Params_t *aParams)
{
	SPI_Obj_t *obj = (SPI_Obj_t *)aHandle;

	EALLOW;

	switch(obj->port)
	{
		default:
			PLX_ASSERT(0);
			break;
		case 1:
			obj->portHandle = (uint32_t)&SpiaRegs;
			// enable clock to SPIA
			CpuSysRegs.PCLKCR8.bit.SPI_A = 1;
			switch(aPinSet)
			{
				default:
					PLX_ASSERT(0);
					break;
				case 10:
					// configure pins
				    GpioCtrlRegs.GPAGMUX2.bit.GPIO16 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 1;
					GpioCtrlRegs.GPAGMUX2.bit.GPIO17 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 1;
					GpioCtrlRegs.GPBGMUX2.bit.GPIO56 = 0;
					GpioCtrlRegs.GPBMUX2.bit.GPIO56 = 1;
					GpioCtrlRegs.GPBGMUX2.bit.GPIO57 = 0;
					GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 1;
					// enable pull-ups
					GpioCtrlRegs.GPAPUD.bit.GPIO16 = 0;   // Enable pull-up on GPIO16 (SPISIMO-A)
					GpioCtrlRegs.GPAPUD.bit.GPIO17 = 0;   // Enable pull-up on GPIO17 (SPISOMI-A)
					GpioCtrlRegs.GPBPUD.bit.GPIO56 = 0;   // Enable pull-up on GPIO56 (SPICLK-A)
					GpioCtrlRegs.GPBPUD.bit.GPIO57 = 0;   // Enable pull-up on GPIO57 (SPISTE-A)
					// async inputs
					GpioCtrlRegs.GPAQSEL2.bit.GPIO16 = 3; // Asynch input GPIO16 (SPISIMO-A)
					GpioCtrlRegs.GPAQSEL2.bit.GPIO17 = 3; // Asynch input GPIO17 (SPISOMI-A)
					GpioCtrlRegs.GPBQSEL2.bit.GPIO56 = 3; // Asynch input GPIO56 (SPICLK-A)
					GpioCtrlRegs.GPBQSEL2.bit.GPIO57 = 3; // Asynch input GPIO57 (SPISTE-A)
					break;
			}
			break;

		case 2:
			obj->portHandle = (uint32_t)&SpibRegs;
			// enable clock to SPIB
			CpuSysRegs.PCLKCR8.bit.SPI_B = 1;
			switch(aPinSet)
			{
				default:
					PLX_ASSERT(0);
					break;
				case 20:
					// configure pins
				    GpioCtrlRegs.GPAGMUX2.bit.GPIO24 = 1;
					GpioCtrlRegs.GPAMUX2.bit.GPIO24 = 2;
					GpioCtrlRegs.GPAGMUX2.bit.GPIO31 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO31 = 3;
                    GpioCtrlRegs.GPAAMSEL.bit.GPIO22 = 0; //Disable internal DC/DC (default configuration), using this pin as normal GPIO
					GpioCtrlRegs.GPAGMUX2.bit.GPIO22 = 1;
					GpioCtrlRegs.GPAMUX2.bit.GPIO22 = 2;
					GpioCtrlRegs.GPBGMUX1.bit.GPIO33 = 0;
					GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 3;
					// enable pull-ups
					GpioCtrlRegs.GPAPUD.bit.GPIO24 = 0;  // Enable pull-up on GPIO24 (SPISIMO-B)
					GpioCtrlRegs.GPAPUD.bit.GPIO31 = 0;  // Enable pull-up on GPIO31 (SPISOMI-B)
					GpioCtrlRegs.GPAPUD.bit.GPIO22 = 0;  // Asynch input GPIO22 (SPICLK-B)
					GpioCtrlRegs.GPBPUD.bit.GPIO33 = 0;  // Enable pull-up on GPIO33 (SPISTE-B)
					// async inputs
					GpioCtrlRegs.GPAQSEL2.bit.GPIO24 = 3; // Asynch input GPIO24 (SPISIMO-B)
					GpioCtrlRegs.GPAQSEL2.bit.GPIO31 = 3; // Asynch input GPIO31 (SPISOMI-B)
					GpioCtrlRegs.GPAQSEL2.bit.GPIO22 = 3; // Asynch input GPIO22 (SPICLK-B)
					GpioCtrlRegs.GPBQSEL1.bit.GPIO33 = 3; // Asynch input GPIO33 (SPISTE-B)
					break;
			}
			break;
	}

	SPI_REGS_PTR->SPICCR.bit.SPISWRESET = 0;
	SPI_REGS_PTR->SPIFFTX.all = 0x8000;
	SPI_REGS_PTR->SPIPRI.all = 0x0020; // SPI priority high, emulator immediate on
	SPI_REGS_PTR->SPISTS.all = 0x00E0; // Clear all status bits
	SPI_REGS_PTR->SPIBRR.all =  (Uint16)(obj->clk / aParams->BAUDRATE - 1);//Calculate BRR = (LSPCLK freq / SPI CLK freq) - 1
	SPI_REGS_PTR->SPICTL.bit.MASTER_SLAVE = aParams->MASTERSLAVE;
	SPI_REGS_PTR->SPICTL.bit.TALK = 1;
	SPI_REGS_PTR->SPICTL.bit.CLK_PHASE = aParams->CLKPHASE;
	SPI_REGS_PTR->SPICCR.bit.CLKPOLARITY = aParams->CLKPOLARITY;
	SPI_REGS_PTR->SPICCR.bit.SPICHAR = aParams->SPICHAR - 1;
	SPI_REGS_PTR->SPICCR.bit.SPISWRESET = 1;

	EDIS;
}

