/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "F2837xS_EPwm_defines.h"
#ifndef PWM_IMPL_H_
#define PWM_IMPL_H_

typedef enum PWM_UNIT {
	PWM_EPWM_NONE=0,
	PWM_EPWM1,
	PWM_EPWM2,
	PWM_EPWM3,
	PWM_EPWM4,
	PWM_EPWM5,
	PWM_EPWM6,
	PWM_EPWM7,
	PWM_EPWM8,
	PWM_EPWM9,
	PWM_EPWM10,
	PWM_EPWM11,
	PWM_EPWM12
} PWM_Unit_t;

typedef struct PWM_PARAMS {
	Uint16 TBPRD;
	union TBPHS_REG  TBPHS;
	union TBCTL_REG TBCTL;
	union CMPCTL_REG CMPCTL;
	union AQCTLA_REG AQCTLA;
	union DBCTL_REG DBCTL;
	union DBFED_REG DBFED;
	union DBRED_REG DBRED;
	union TZCTL_REG TZCTL;
	union ETSEL_REG ETSEL;
	union TZSEL_REG TZSEL;
} PWM_Params_t;

typedef struct PWM_OBJ
{
	volatile struct EPWM_REGS *pwm;
} PWM_Obj_t;

typedef PWM_Obj_t *PWM_Handle_t;

extern void PWM_getRegisterBase(PWM_Unit_t aPwmChannel, volatile struct EPWM_REGS** aReg);

inline uint32_t PWM_getPeriod(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    if(obj->pwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN)
    {
        // up-down
		return obj->pwm->TBPRD;
	}
    else
    {
        // triangle
        return obj->pwm->TBPRD+1;
    }
}

inline uint32_t PWM_getCounter(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	return obj->pwm->TBCTR;
}

inline bool PWM_getCountDirection(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	return obj->pwm->TBSTS.bit.CTRDIR;
}

inline void PWM_enableOut(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	EALLOW;
	obj->pwm->TZCLR.bit.OST=1;
	EDIS;
}

inline void PWM_disableOut(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	EALLOW;
	obj->pwm->TZFRC.bit.OST=1;
	EDIS;
}

inline void PWM_setCompare(PWM_Handle_t aHandle, uint16_t aCompare)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->CMPA.bit.CMPA = aCompare;
}

inline void PWM_setTZSafe(PWM_Handle_t aHandle, uint16_t aSafe)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	obj->pwm->TZCTL.bit.TZA = aSafe;
	obj->pwm->TZCTL.bit.TZB = aSafe;
}

inline void PWM_setComparePh(PWM_Handle_t aHandle, uint16_t aCompare, uint16_t aPh, uint16_t aDir)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->CMPA.bit.CMPA = aCompare;
    obj->pwm->TBPHS.bit.TBPHS= aPh;
    obj->pwm->TBCTL.bit.PHSDIR = aDir;
}

#endif /* PWM_IMPL_H_ */
