/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "F2837xS_Adc_defines.h"
#include "ain.h"

#pragma diag_suppress 112 // PLX_ASSERT(0) in switch statements

/* Note:
 * No power-up delay implemented. Assuming system-level delay will allow ADC to settle
 */

extern void AdcSetMode(Uint16 adc, Uint16 resolution, Uint16 signalmode);

void AIN_sinit()
{
	;
}

void AIN_getRegisterBase(AIN_Unit_t aAdcUnit, volatile struct ADC_REGS** aReg){
	switch(aAdcUnit)
	{
	default:
		PLX_ASSERT(0);
		break;

	case AIN_ADC_A:
		*aReg = &AdcaRegs;
		break;

	case AIN_ADC_B:
		*aReg = &AdcbRegs;
		break;

	case AIN_ADC_C:
		*aReg = &AdccRegs;
		break;

	case AIN_ADC_D:
		*aReg = &AdcdRegs;
		break;
	}
}

void AIN_setDefaultAdcParams(AIN_AdcParams_t *aParams)
{
	aParams->ADCCTL1.bit.INTPULSEPOS = 1; // set pulse positions to late
	// max allowable ADC clock is 50 MHz (min 5 MHz)
#if(SYSCLK_HZ <= 100000000L)
	aParams->ADCCTL2.bit.PRESCALE = 2; //set ADCCLK divider to /2.0
#elif(SYSCLK_HZ <= 120000000L)
	aParams->ADCCTL2.bit.PRESCALE = 3; //set ADCCLK divider to /2.5
#elif(SYSCLK_HZ <= 150000000L)
	aParams->ADCCTL2.bit.PRESCALE = 4; //set ADCCLK divider to /3.0
#elif(SYSCLK_HZ <= 175000000L)
	aParams->ADCCTL2.bit.PRESCALE = 5; //set ADCCLK divider to /3.5
#elif(SYSCLK_HZ <= 200000000L)
	aParams->ADCCTL2.bit.PRESCALE = 6; //set ADCCLK divider to /4.0
#else
#error Invalid clock frequency
#endif
	aParams->ADCSOCPRICTL.bit.SOCPRIORITY = 0; // round-robin - no high priority channels
}

AIN_Handle_t AIN_init(void *aMemory, const size_t aNumBytes)
{
	AIN_Handle_t handle;

	if(aNumBytes < sizeof(AIN_Obj_t))
		return((AIN_Handle_t)NULL);

	// set handle
	handle = (AIN_Handle_t)aMemory;

	return handle;
}

void AIN_configure(AIN_Handle_t aHandle, AIN_Unit_t aUnit, const AIN_AdcParams_t *aParams)
{
	AIN_Obj_t *obj = (AIN_Obj_t *)aHandle;

	EALLOW;
	switch(aUnit)
	{
		default:
			PLX_ASSERT(0);
			break;

		case AIN_ADC_A:
			obj->adc = &AdcaRegs;
			obj->results = (uint16_t *)&AdcaResultRegs;
			CpuSysRegs.PCLKCR13.bit.ADC_A = 1;
		    AdcSetMode(ADC_ADCA, ADC_RESOLUTION_12BIT, ADC_SIGNALMODE_SINGLE);
			break;

		case AIN_ADC_B:
			obj->adc = &AdcbRegs;
			obj->results = (uint16_t *)&AdcbResultRegs;
			CpuSysRegs.PCLKCR13.bit.ADC_B = 1;
		    AdcSetMode(ADC_ADCB, ADC_RESOLUTION_12BIT, ADC_SIGNALMODE_SINGLE);
			break;

		case AIN_ADC_C:
			obj->adc = &AdccRegs;
			obj->results = (uint16_t *)&AdccResultRegs;
			CpuSysRegs.PCLKCR13.bit.ADC_C = 1;
		    AdcSetMode(ADC_ADCC, ADC_RESOLUTION_12BIT, ADC_SIGNALMODE_SINGLE);
			break;

		case AIN_ADC_D:
			obj->adc = &AdcdRegs;
			obj->results = (uint16_t *)&AdcdResultRegs;
			CpuSysRegs.PCLKCR13.bit.ADC_D = 1;
		    AdcSetMode(ADC_ADCD, ADC_RESOLUTION_12BIT, ADC_SIGNALMODE_SINGLE);
			break;
	}
	EDIS;

	obj->socCtrl = (union ADCSOC0CTL_REG *)&obj->adc->ADCSOC0CTL;

	EALLOW;
	obj->adc->ADCCTL1.bit.ADCPWDNZ = 1; // power up the ADC
	obj->adc->ADCCTL1.bit.INTPULSEPOS = aParams->ADCCTL1.bit.INTPULSEPOS;
	obj->adc->ADCCTL2.bit.PRESCALE = aParams->ADCCTL2.bit.PRESCALE;
	obj->adc->ADCSOCPRICTL.bit.SOCPRIORITY = aParams->ADCSOCPRICTL.bit.SOCPRIORITY;
	EDIS;
	AIN_resetChannelSetup(aHandle);
}

void AIN_setDefaultChannelParams(AIN_ChannelParams_t *aParams)
{
	aParams->ADCSOCxCTL.bit.TRIGSEL = 1; // trigger on CpuTimer0
	// minimal sample window is for single-ended is 75 ns, determined by SYSCLK
#if(SYSCLK_HZ <= 100000000L)
	aParams->ADCSOCxCTL.bit.ACQPS = 7;
#elif(SYSCLK_HZ <= 120000000L)
	aParams->ADCSOCxCTL.bit.ACQPS = 8;
#elif(SYSCLK_HZ <= 150000000L)
	aParams->ADCSOCxCTL.bit.ACQPS = 11;
#elif(SYSCLK_HZ <= 175000000L)
	aParams->ADCSOCxCTL.bit.ACQPS = 13;
#elif(SYSCLK_HZ <= 200000000L)
	aParams->ADCSOCxCTL.bit.ACQPS = 14;
#else
#error Invalid clock frequency
#endif
}

void AIN_resetChannelSetup(AIN_Handle_t aHandle)
{
	AIN_Obj_t *obj = (AIN_Obj_t *)aHandle;

	int i;
	EALLOW;
	for(i=0; i<AIN_NUM_CHANNELS; i++)
	{
		obj->socCtrl[i].bit.TRIGSEL = 0;
	}
	EDIS;
}

void AIN_setupChannel(AIN_Handle_t aHandle, uint16_t aChannel, uint16_t aSource, const AIN_ChannelParams_t *aParams)
{
	AIN_Obj_t *obj = (AIN_Obj_t *)aHandle;

	PLX_ASSERT(aChannel < AIN_NUM_CHANNELS);
	PLX_ASSERT(aParams->ADCSOCxCTL.bit.TRIGSEL <= 0x1C);

	EALLOW;
	obj->socCtrl[aChannel].bit.CHSEL = aSource;
	obj->socCtrl[aChannel].bit.TRIGSEL = aParams->ADCSOCxCTL.bit.TRIGSEL;
	obj->socCtrl[aChannel].bit.ACQPS = aParams->ADCSOCxCTL.bit.ACQPS;
	EDIS;
}
