/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "F2837xS_EPwm_defines.h"
#include "pwm.h"
#include "F2837xS_Gpio_defines.h"

/* TODO: Support all PWM channels! */

#pragma diag_suppress 112 // PLX_ASSERT(0) in switch statements

void PWM_sinit()
{
	EALLOW;
	CpuSysRegs.PCLKCR2.bit.EPWM1 = 1; // ePWM1
	CpuSysRegs.PCLKCR2.bit.EPWM2 = 1; // ePWM2
	CpuSysRegs.PCLKCR2.bit.EPWM3 = 1; // ePWM3
	CpuSysRegs.PCLKCR2.bit.EPWM4 = 1; // ePWM4
	CpuSysRegs.PCLKCR2.bit.EPWM5 = 1; // ePWM5
	CpuSysRegs.PCLKCR2.bit.EPWM6 = 1; // ePWM6
	CpuSysRegs.PCLKCR2.bit.EPWM7 = 1; // ePWM7
	CpuSysRegs.PCLKCR2.bit.EPWM8 = 1; // ePWM8
	CpuSysRegs.PCLKCR2.bit.EPWM9 = 1; // ePWM9
	CpuSysRegs.PCLKCR2.bit.EPWM10 = 1; // ePWM10
	CpuSysRegs.PCLKCR2.bit.EPWM11 = 1; // ePWM11
	CpuSysRegs.PCLKCR2.bit.EPWM12 = 1; // ePWM12
#if (SYSCLK_HZ <= 100000000L)
	ClkCfgRegs.PERCLKDIVSEL.bit.EPWMCLKDIV = 0; // do not divide clock by 2
#else
	ClkCfgRegs.PERCLKDIVSEL.bit.EPWMCLKDIV = 1; // divide clock by 2 to respect 100 MHz max
#endif

	EPwm1Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm2Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm3Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm4Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm5Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm6Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm7Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm8Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm9Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm10Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm11Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EPwm12Regs.TBCTL.bit.SYNCOSEL = TB_SYNC_IN; // sync flow-through
	EDIS;
}

void PWM_getRegisterBase(PWM_Unit_t aPwmChannel, volatile struct EPWM_REGS** aReg){
	switch(aPwmChannel)
	{
		default:
			PLX_ASSERT(0);
			break;
		case PWM_EPWM1:
			*aReg = &EPwm1Regs;
			break;
		case PWM_EPWM2:
			*aReg = &EPwm2Regs;
			break;
		case PWM_EPWM3:
			*aReg = &EPwm3Regs;
			break;
		case PWM_EPWM4:
			*aReg = &EPwm4Regs;
			break;
		case PWM_EPWM5:
			*aReg = &EPwm5Regs;
			break;
		case PWM_EPWM6:
			*aReg = &EPwm6Regs;
			break;
		case PWM_EPWM7:
			*aReg = &EPwm7Regs;
			break;
		case PWM_EPWM8:
			*aReg = &EPwm8Regs;
			break;
		case PWM_EPWM9:
			*aReg = &EPwm9Regs;
			break;
		case PWM_EPWM10:
			*aReg = &EPwm10Regs;
			break;
		case PWM_EPWM11:
			*aReg = &EPwm11Regs;
			break;
		case PWM_EPWM12:
			*aReg = &EPwm12Regs;
			break;
	}
}

void PWM_setDefaultParams(PWM_Params_t *aParams)
{
	aParams->TBPRD = 0;
	aParams->DBFED.all = 0;
	aParams->DBRED.all = 0;
	aParams->TBCTL.bit.SYNCOSEL = TB_SYNC_IN;

	aParams->TBPHS.bit.TBPHS = 0; // set Phase register to zero
	aParams->TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; // symmetrical mode
	aParams->TBCTL.bit.PHSEN = TB_DISABLE; // master module
	aParams->TBCTL.bit.PRDLD = TB_SHADOW;
	aParams->CMPCTL.bit.SHDWAMODE = CC_SHADOW;
	aParams->CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    aParams->CMPCTL.bit.LOADAMODE = CC_CTR_ZERO_PRD; // load on CTR=Zero and Prd
    aParams->CMPCTL.bit.LOADBMODE = CC_CTR_ZERO_PRD; // load on CTR=Zero and Prd
    aParams->AQCTLA.bit.CAU = AQ_SET; // set actions for EPWM1A
    aParams->AQCTLA.bit.CAD = AQ_CLEAR;
    aParams->AQCTLA.bit.PRD = AQ_NO_ACTION;
    aParams->AQCTLA.bit.ZRO = AQ_NO_ACTION;
	aParams->DBCTL.bit.OUT_MODE = DB_FULL_ENABLE; // enable Dead-band module
	aParams->DBCTL.bit.POLSEL = DB_ACTV_HIC; // active Hi complementary
	aParams->TZCTL.bit.TZA = TZ_NO_CHANGE;
	aParams->TZCTL.bit.TZB = TZ_NO_CHANGE;

	aParams->ETSEL.bit.SOCASEL = ET_CTR_ZERO;
	aParams->ETSEL.bit.INTSEL = ET_CTR_ZERO;
}

PWM_Handle_t PWM_init(void *aMemory, const size_t aNumBytes)
{
	PWM_Handle_t handle;

	if(aNumBytes < sizeof(PWM_Obj_t))
		return((PWM_Handle_t)NULL);

	// set handle
	handle = (PWM_Handle_t)aMemory;

	return handle;
}

void PWM_configure(PWM_Handle_t aHandle, uint16_t aModulator, const PWM_Params_t *aParams)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;

	PLX_ASSERT(aModulator >= 1);
	PLX_ASSERT(aModulator <= 12);

	// TODO: Use PWM_getRegisterBase
	switch(aModulator)
	{
		default:
		case 1:
			obj->pwm = &EPwm1Regs;
			break;
		case 2:
			obj->pwm = &EPwm2Regs;
			break;
		case 3:
			obj->pwm = &EPwm3Regs;
			break;
		case 4:
			obj->pwm = &EPwm4Regs;
			break;
		case 5:
			obj->pwm = &EPwm5Regs;
			break;
		case 6:
			obj->pwm = &EPwm6Regs;
			break;
		case 7:
			obj->pwm = &EPwm7Regs;
			break;
		case 8:
			obj->pwm = &EPwm8Regs;
			break;
		case 9:
			obj->pwm = &EPwm9Regs;
			break;
		case 10:
			obj->pwm = &EPwm10Regs;
			break;
		case 11:
			obj->pwm = &EPwm11Regs;
			break;
		case 12:
			obj->pwm = &EPwm12Regs;
			break;
	}

	EALLOW;

	obj->pwm->TBPRD = aParams->TBPRD;
	obj->pwm->DBFED.all = aParams->DBFED.all;
	obj->pwm->DBRED.all = aParams->DBRED.all;
	obj->pwm->TBCTL.bit.SYNCOSEL = aParams->TBCTL.bit.SYNCOSEL;

	obj->pwm->TBPHS.bit.TBPHS = aParams->TBPHS.bit.TBPHS;
	obj->pwm->TBCTL.bit.CTRMODE = aParams->TBCTL.bit.CTRMODE;
	obj->pwm->TBCTL.bit.PHSEN = aParams->TBCTL.bit.PHSEN;
	obj->pwm->TBCTL.bit.PRDLD = aParams->TBCTL.bit.PRDLD;
	obj->pwm->CMPCTL.bit.SHDWAMODE = aParams->CMPCTL.bit.SHDWAMODE;
	obj->pwm->CMPCTL.bit.SHDWBMODE = aParams->CMPCTL.bit.SHDWBMODE;
    obj->pwm->CMPCTL.bit.LOADAMODE = aParams->CMPCTL.bit.LOADAMODE;
    obj->pwm->CMPCTL.bit.LOADBMODE = aParams->CMPCTL.bit.LOADBMODE;
	obj->pwm->AQCTLA.bit.CAU = aParams->AQCTLA.bit.CAU;
	obj->pwm->AQCTLA.bit.CAD = aParams->AQCTLA.bit.CAD;
	obj->pwm->AQCTLA.bit.PRD = aParams->AQCTLA.bit.PRD;
	obj->pwm->AQCTLA.bit.ZRO = aParams->AQCTLA.bit.ZRO;
	obj->pwm->DBCTL.bit.OUT_MODE = aParams->DBCTL.bit.OUT_MODE;
	obj->pwm->DBCTL.bit.POLSEL = aParams->DBCTL.bit.POLSEL;
	obj->pwm->TZCTL.bit.TZA = aParams->TZCTL.bit.TZA;
	obj->pwm->TZCTL.bit.TZB = aParams->TZCTL.bit.TZB;
	obj->pwm->TZSEL.bit.CBC1 = aParams->TZSEL.bit.CBC1;
	obj->pwm->TZSEL.bit.CBC2 = aParams->TZSEL.bit.CBC2;
	obj->pwm->TZSEL.bit.CBC3 = aParams->TZSEL.bit.CBC3;
	obj->pwm->TZSEL.bit.OSHT1 = aParams->TZSEL.bit.OSHT1;
	obj->pwm->TZSEL.bit.OSHT2 = aParams->TZSEL.bit.OSHT2;
	obj->pwm->TZSEL.bit.OSHT3 = aParams->TZSEL.bit.OSHT3;

	obj->pwm->ETSEL.bit.SOCASEL = aParams->ETSEL.bit.SOCASEL;
	obj->pwm->ETSEL.bit.INTSEL = aParams->ETSEL.bit.INTSEL;

	// make sure all is off
	obj->pwm->TZFRC.bit.OST=1;

	switch(aModulator)
	{
		default:
		case 1:
			GpioCtrlRegs.GPAPUD.bit.GPIO0 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO0 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO1 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO1 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 1;
		    break;
		case 2:
			GpioCtrlRegs.GPAPUD.bit.GPIO2 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO2 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO3 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO3 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 1;
			break;
		case 3:
			GpioCtrlRegs.GPAPUD.bit.GPIO4 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO4 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO5 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO5 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 1;
		    break;
		case 4:
			GpioCtrlRegs.GPAPUD.bit.GPIO6 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO6 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO7 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO7 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 1;
			break;
		case 5:
			GpioCtrlRegs.GPAPUD.bit.GPIO8 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO8 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO8 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO9 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO9 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO9 = 1;
			break;
		case 6:
			GpioCtrlRegs.GPAPUD.bit.GPIO10 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO10 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO11 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO11 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
			break;
		case 7:
			GpioCtrlRegs.GPAPUD.bit.GPIO12 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO12 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO13 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO13 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;
			break;
		case 8:
			GpioCtrlRegs.GPAPUD.bit.GPIO14 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO14 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO14 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO15 = 1;
		    GpioCtrlRegs.GPAGMUX1.bit.GPIO15 = 0;
		    GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 1;
			break;
		case 9:
			GpioCtrlRegs.GPAPUD.bit.GPIO16 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO16 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO16 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO17 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO17 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO17 = 1;
			break;
		case 10:
			GpioCtrlRegs.GPAPUD.bit.GPIO18 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO18 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO19 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO19 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 1;
			break;
		case 11:
			GpioCtrlRegs.GPAPUD.bit.GPIO20 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO20 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO21 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO21 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 1;
			break;
		case 12:
			GpioCtrlRegs.GPAPUD.bit.GPIO22 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO22 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO22 = 1;
			GpioCtrlRegs.GPAPUD.bit.GPIO23 = 1;
		    GpioCtrlRegs.GPAGMUX2.bit.GPIO23 = 1;
		    GpioCtrlRegs.GPAMUX2.bit.GPIO23 = 1;
			break;
	}

	EDIS;
}

void PWM_configureSyncSel(uint16_t aResourceNumber, uint16_t aType, uint16_t aUnit)
{
	EALLOW;

	if (aType == 1)
	{
		// determine registers and masks
		uint16_t group = aUnit / 32;
		uint16_t groupIndex = aUnit % 32; // pin32
		uint16_t mux = groupIndex / 16;
		uint16_t muxIndex = groupIndex % 16; // pin 16
		uint32_t mask = 1L << groupIndex;
		uint32_t muxMask = 3L << (muxIndex * 2);

		uint32_t *ctrlRegsBaseAddr = (uint32_t *)&GpioCtrlRegs + group*GPY_CTRL_OFFSET;
		uint32_t *dirReg = ctrlRegsBaseAddr + GPYDIR;
		uint32_t *muxReg = ctrlRegsBaseAddr + GPYMUX + mux;
		uint32_t *muxGReg = ctrlRegsBaseAddr + GPYGMUX + mux;

		// configure hardware
		*muxReg &= ~muxMask;
		*muxGReg &= ~muxMask;
		*dirReg &= mask;
	}

	switch(aResourceNumber)
	{
		case 1:
			if (aType == 0)
			{

			}
			else
			{
				InputXbarRegs.INPUT5SELECT = aUnit; // EXTSYNC1 !!!
			}
			break;
		case 4:
			if (aType == 0)
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM4SYNCIN = aUnit;
			}
			else
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM4SYNCIN = 5;
				InputXbarRegs.INPUT5SELECT = aUnit; // EXTSYNC1 !!!
			}
			break;
		case 7:
			if (aType == 0)
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM7SYNCIN = aUnit;
			}
			else
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM7SYNCIN = 5;
				InputXbarRegs.INPUT5SELECT = aUnit; // EXTSYNC1 !!!
			}
			break;
		case 10:
			if (aType == 0)
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM10SYNCIN = aUnit;
			}
			else
			{
				SyncSocRegs.SYNCSELECT.bit.EPWM10SYNCIN = 5;
				InputXbarRegs.INPUT5SELECT = aUnit; // EXTSYNC1 !!!
			}
			break;
		default:
			break;
	}

	EDIS;
}
